﻿var cph5 = cph5||{};

(function (cph5) {
	
    var Touch = (function () {
        function Touch(stage, canvas, touchBegan, touchMove, touchEnd) {
            this._stage  = stage;
            this._canvas = canvas;
            this._isTouchDown = false;
            this._lastTouchX = -1;
            this._lastTouchY = -1;
            this._touchBegan = touchBegan;
            this._touchMove = touchMove;
            this._touchEnd = touchEnd;
        }
        
        Touch.prototype.start = function () {
            var that = this;
            
            this.addTouchListener();
          	this.addMouseListener();

            window.addEventListener("mousedown", function (event) {
                if (!that.inOutOfCanvas(event)) {
                    that._isTouchDown = true;
                }
            });

            window.addEventListener("mouseup", function (event) {
                that._isTouchDown = false;
            });
        };

        Touch.prototype.addMouseListener = function () {
            var that = this;
            this._canvas.addEventListener("mousedown", function (event) {
                that._onTouchBegin(event);
            });
            this._canvas.addEventListener("mousemove", function (event) {
                that._onTouchMove(event);
            });
            this._canvas.addEventListener("mouseup", function (event) {
                that._onTouchEnd(event);
            });
        };

        Touch.prototype.addTouchListener = function () {
            var that = this;
            this._canvas.addEventListener("touchstart", function (event) {
                var l = event.changedTouches.length;
                for (var i = 0; i < l; i++) {
                    that._onTouchBegin(event.changedTouches[i]);
                }
            }, false);
            
            this._canvas.addEventListener("touchmove", function (event) {
                var l = event.changedTouches.length;
                for (var i = 0; i < l; i++) {
                    that._onTouchMove(event.changedTouches[i]);
                }
            }, false);            
            this._canvas.addEventListener("touchend", function (event) {
                var l = event.changedTouches.length;
                for (var i = 0; i < l; i++) {
                    that._onTouchEnd(event.changedTouches[i]);
                }
            }, false);
            this._canvas.addEventListener("touchcancel", function (event) {
                var l = event.changedTouches.length;
                for (var i = 0; i < l; i++) {
                    that._onTouchEnd(event.changedTouches[i]);
                }
            }, false);
        };

        Touch.prototype.inOutOfCanvas = function (event) {
            var location = this.getLocation(this._canvas, event);
            if (location.x < 0 || location.y < 0 || location.x > this._canvas.width || location.y > this._canvas.height) {
                return true;
            }
            return false;
        };

        Touch.prototype._onTouchBegin = function (event) {
            var location = this.getLocation(this._canvas, event);
            this.onTouchBegan(location.x, location.y);
        };

        Touch.prototype._onTouchMove = function (event) {
            var location = this.getLocation(this._canvas, event);
            this.onTouchMove(location.x, location.y);
        };

        Touch.prototype._onTouchEnd = function (event) {
            var location = this.getLocation(this._canvas, event);
            this.onTouchEnd(location.x, location.y);
        };

        Touch.prototype.onTouchBegan = function (x,y) {
            if (this._touchBegan) this._touchBegan(x,y);
        };

        Touch.prototype.onTouchMove = function (x,y) {
            if (this._touchMove) this._touchMove(x,y);
        };

        Touch.prototype.onTouchEnd = function (x,y) {
            if (this._touchEnd) this._touchEnd(x,y);
        };

        Touch.prototype.getLocation = function (canvas, event) {
            var doc = document.documentElement;
            var win = window;
            var left, top, tx, ty;

            if (typeof canvas.getBoundingClientRect === 'function') {
                var box = canvas.getBoundingClientRect();
                left = box.left;
                top = box.top;
            } else {
                left = 0;
                top = 0;
            }

            left += win.pageXOffset - doc.clientLeft;
            top += win.pageYOffset - doc.clientTop;

            if (event.pageX != null) {
                tx = event.pageX;
                ty = event.pageY;
            } else {
                left -= document.body.scrollLeft;
                top -= document.body.scrollTop;
                tx = event.clientX;
                ty = event.clientY;
            }
            var result = {};
            result.x = (tx - left) / this._stage._scaleX;
            result.y = (ty - top) / this._stage._scaleY;
            return result;
        };
        return Touch;
    })();
    cph5.Touch = Touch;
    Touch.prototype.__class__ = "cph5.Touch";
    
})(cph5);
