﻿var cph5 = cph5 || {};

(function (cph5) {
	
    var Stage = (function () {
        function Stage(container, canvas, fps, designWidth, designHeight) {
        		if (fps < 1)
        			fps = 1;
 
						this._designWidth = designWidth;
						this._designHeight = designHeight;
 
 						this._container = container;
        		this._canvas		= canvas;
        		this._context 	= canvas.getContext("2d");

 						this.onResizeEvent();

        		this._frameRate = fps;
        		this._timeout		= 1000.0/fps;
            this._interal		= 0;
        		this._time 			= Date.now();
        		
            this._touchBegan 	= null;
            this._touchMove 	= null;
            this._touchEnd 	= null;
            this._update = null;
            this._draw 	= null;
				
            this._stop			= false;
            
            this._touch			= null;
            
            var that = this;
            window.addEventListener("resize", function (event) {
                that.onResizeEvent();
            });

        }
        
        Stage.prototype.stop = function() {
        	this._stop			= true;
        	if (this._interal != 0) {
        		window.clearTimeout(this._interal);
        		this._interal = 0;
        	}
        }
        
        Stage.prototype.start = function() {
						var that 			= this;
						this._stop		= false;
						this._time 		= Date.now();
						
						this._touch 	= new cph5.Touch(this, this._canvas, this._touchBegan, this._touchMove, this._touchEnd);
						this._touch.start();
						
						this._interal = window.setTimeout(onTimeout, this._timeout);
						function onTimeout() {
							var time = Date.now();
							var dt 	 = time - that._time;
							that._time = time;
							that._enterFrame(dt);
							if (!that._stop)
								that._interal = window.setTimeout(onTimeout, that._timeout);
						}
        }
        
        Stage.prototype._enterFrame = function(delta) {
        	if (this._update) this._update(delta);
        	
        	if (this._draw) {
        		this._context.clearRect(0, 0, this._canvas.width, this._canvas.height);  
        		this._draw(this._context);
	        }
        }

        Stage.prototype.onResizeEvent = function() {
						var documentWidth 	= document.documentElement.clientWidth;
						var documentHeight 	= document.documentElement.clientHeight;
						if (documentWidth<=0||documentHeight<=0) {
							documentWidth 	= this._designWidth;
							documentHeight	= this._designHeight;
						}
						var scale 					= Math.min(documentWidth/this._designWidth, documentHeight/this._designHeight);
						var viewPortWidth 	= this._designWidth  * scale;
						var viewPortHeight 	= this._designHeight * scale;
			
						this._canvas.width 						= this._designWidth;
						this._canvas.height 					= this._designHeight;
						this._canvas.style.width 			= viewPortWidth  + "px";
						this._canvas.style.height 		= viewPortHeight + "px";
						this._canvas.style.left 			= Math.floor((documentWidth  - viewPortWidth ) / 2) + "px";
						this._canvas.style.top 				= Math.floor((documentHeight - viewPortHeight) / 2) + "px";
						this._container.style.width 	= viewPortWidth  + "px";
						this._container.style.height 	= viewPortHeight + "px";
						
						this._scaleX = scale;
						this._scaleY = scale;
        }

        return Stage;
    })();
    
    cph5.Stage = Stage;
    Stage.prototype.__class__ = "cph5.Stage";
	
})(cph5);
